"""
Name: GAssist_Constants.py
Author: Ryan Urbanowicz - written at Dartmouth College, Hanover, NH, USA 
Note: code primarily based on Java GAssist authored by Xavier Llor <xllora@illigal.ge.uiuc.edu>
Contact: ryan.j.urbanowicz@darmouth.edu
Created: 2/22/2010
Updated: 10/15/2013
Description: Specifies default GAssist run constants.
---------------------------------------------------------------------------------------------------------------------------------------------------------
GAssist Version 1.0: A Python implementation of the GAssist learning classifier system algorithm implemented for SNP genetic data mining and classification tasks.
Copyright (C) 2013 Ryan Urbanowicz 
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABLILITY 
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, 
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
---------------------------------------------------------------------------------------------------------------------------------------------------------
"""

#!/usr/bin/env python

#Import modules
from GAssist_Environment import *

class CGAssistConstants:
    """ Specifies default GAssist run constants. """

    #Initialization Parameters
    agentInitWidth = 20 # Initial complexity of agent    
    dontCare = '#'
    
    #Major Architectural Parameters
    doRuleDeletion = False #was self.
    doHierarchicalSelection = False #was self.

    #Other Pressure Parameters  
    hierarchicalSelectionThreshold = 0
    iterationHierarchicalSelection = 24 
    iterationRuleDeletion = 5
    sizePenaltyMinRules = 4
    iterationMDL = 25 #the iteration where MDL kicks in.
    initialTheoryLengthRatio = 0.075 #used in calculating the theory weight
    weightRelaxFactor = 0.90 
    ruleDeletionMinRules = 12
    
    tournamentSize = 3 #in actuality the T-size is the value entered here + 1 according to the coding.
    probCrossover = 0.6 #probability of a crossover
    probMutationInd = 0.6 

    
    def setConstants(self, popSize, wild, defaultClass, env, MDL, windows):
        """ Initialize user defined constants. """
        self.popSize = popSize
        self.wild = wild
        self.numStrata = int(windows)
        if int(MDL) == 0:
            self.useMDL = False
        elif int(MDL) == 1:
            self.useMDL = True
        else:
            self.useMDL = -1
            print "Incorrect MDL setting selected."
            
        if defaultClass == "auto":
            self.defaultClass = str(defaultClass)
            self.nichingEnabled = True
            # Tracks last 15 average accuracies for each niche. (only needed with niching)
            self.accDefaultRules = [[] for i in range(env.getNrActions())] #hope this works. - nope it doesn't
        elif defaultClass == "disabled":
            self.defaultClass = str(defaultClass) 
            self.nichingEnabled = False
        else:
            self.defaultClass = int(defaultClass)
            self.nichingEnabled = False
    
    def setLearning(self, pol):
        """ Sets the learning progress each iteration. """
        self.percentageOfLearning = pol
    
    def setRuleDelete(self, delete):
        self.doRuleDeletion = delete

    def setHierarchy(self, hier):
        self.doHierarchicalSelection = hier
        
    def setDeleteMinRules(self, dmr):
        self.ruleDeletionMinRules = dmr
        
    def setNichingStatus(self, niche):
        self.nichingEnabled = niche

GAssistConstants = CGAssistConstants()
cons = GAssistConstants