"""
Name: GAssist_Global_MDL.py
Author: Ryan Urbanowicz - written at Dartmouth College, Hanover, NH, USA 
Note: code primarily based on Java GAssist authored by Xavier Llor <xllora@illigal.ge.uiuc.edu>
Contact: ryan.j.urbanowicz@darmouth.edu
Created: 2/22/2010
Updated: 10/15/2013
Description: Handles MDL fitness calculations and controls variable activation of fitness calculations.
---------------------------------------------------------------------------------------------------------------------------------------------------------
GAssist Version 1.0: A Python implementation of the GAssist learning classifier system algorithm implemented for SNP genetic data mining and classification tasks.
Copyright (C) 2013 Ryan Urbanowicz 
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABLILITY 
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, 
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
---------------------------------------------------------------------------------------------------------------------------------------------------------
"""

#!/usr/bin/env python

#Import modules
from GAssist_Constants import * 
from GAssist_Environment import *

class GAssistGlobalMDL:
    def __init__(self, env): 
        """ Initialize the global tracking MDL parameters. """
        self.theoryWeight = 0.0
        self.activated = False
        self.fixedWeight = False
        self.e = env
    
    def newIteration(self, iteration, pop):
        """ Called by Timers to alter how fitness is calculated over time. """
        if not cons.useMDL:
            return False
        
        ind = pop.getBest()
        
        updateWeight = False
        if iteration == cons.iterationMDL:
            print "Iteration " + str(iteration) + " MDL fitness activated"
            self.activated = True
            error = ind.getExceptionsLength() 
            theoryLength = ind.getTheoryLength()
            theoryLength *= self.e.getNrActions()
            theoryLength /= ind.agnPer.getNumAliveRules()
            
            self.theoryWeight = (cons.initialTheoryLengthRatio / (1.0 - cons.initialTheoryLengthRatio)) * (error / theoryLength)
            updateWeight = True
            
        if self.activated and not self.fixedWeight and pop.last10IterationsAccuracyAverage == 1.0:  
            print "Fixed Weight activated! = only if perfect accuracy has been obtained."
            self.fixedWeight = True

        if self.activated and not self.fixedWeight: 
            if ind.agnPer.getAccuracy() != 1.0:  
                if pop.getIterationsSinceBest() == 10: #theory weight updated every 10 iterations.
                    self.theoryWeight *= cons.weightRelaxFactor
                    print "New theory weight = " + str(self.theoryWeight)
                    updateWeight = True

        if updateWeight:
            pop.resetBestStats()
            return True
            
        return False
    

    def mdlFitness(self, ind):
        """ Computes the MDL based fitness - which places emphasis on a maximally small/simple and accurate set of rules as making up a most fit agent. """
        mdlFit = 0.0
        ind.computeTheoryLength() 
        if self.activated: 
            mdlFit = ind.getTheoryLength() * self.theoryWeight 
        exceptionsLength = 105.00 - ind.agnPer.getAccuracy() * 100.0 
        ind.setExceptionsLength(exceptionsLength) 
        mdlFit += exceptionsLength
        return mdlFit
    