"""
Name: GAssist_Main.py
Author: Ryan Urbanowicz - written at Dartmouth College, Hanover, NH, USA 
Note: code primarily based on Java GAssist authored by Xavier Llor <xllora@illigal.ge.uiuc.edu>
Contact: ryan.j.urbanowicz@darmouth.edu
Created: 2/22/2010
Updated: 10/15/2013
Description: Command line Main method for running the GAssist algorithm.  Handles inputs, builds the environment, and runs the GAssist algorithm
---------------------------------------------------------------------------------------------------------------------------------------------------------
GAssist Version 1.0: A Python implementation of the GAssist learning classifier system algorithm implemented for SNP genetic data mining and classification tasks.
Copyright (C) 2013 Ryan Urbanowicz 
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABLILITY 
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, 
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
---------------------------------------------------------------------------------------------------------------------------------------------------------
"""

#!/usr/bin/env python

#Import modules
from GAssist import *
from GAssist_Environment import *
import time
import sys
argv = sys.argv

# *************************************************************************************************
helpstr = """
Parameters:
    1:  Specify the environment - 'gh' - gh = genetic heterogeneity snp datasets with discretely coded genotype attributes and a case/control classes.
    2:  Training Dataset - '/SomePath/SomeTrainfile.txt'
    3:  Testing Dataset - '/SomePath/SomeTestfile.txt'
    4:  Output Filename (Learning Progress Tracking) - '/SomePath/SomeProgressName'
    5:  Output Filename (Final Pop Evaluation) - '/SomePath/SomeFinalPopName'
    6:  Coding Length - '1'
    7:  Dataset Partitions for CV - '10'
    8:  Performance Tracking Cycles - '1'
    9:  Learning iterations - '100.500.1000' or for testing '10.20'
    10: Initialization - 'none', 'smart', 'cw'
    11: PopSize - '100, 500, 1000'
    12: Wild frequency - 0.5, 0.75
    13: Default Class Handeling - auto, 0, disabled  - automatic default class evolution, vs. pre-specify the default class (can also add a method which chooses via which class is most represented.
    14: MDL fitness - '0', '1' - where 0 is False, and 1 is True.
    15: Windows - '1', '2', '4' - number of partitions of the dataset used in the learning process. (higher values tend to promote generalization, and reduce computational time.)
"""
# *************************************************************************************************

graphPerformance = False # Built in graphing ability, currently not functional, but mechanism is in place. NOT MEANT TO BE USED ON CLUSTER.
numArgs = len(argv)
print "Arguments: " + str(numArgs)
if numArgs == 16:
    if argv[1] == 'gh': #Different rule representations could be programmed but have not been in this implementation.
        print ("Format Training data: "+argv[2]+"  using a "+argv[6]+" bit coding scheme.")
        
        #Sets up up algorithm to be run.
        e = GAssist_Environment(str(argv[2]), str(argv[3]), int(argv[6]), str(argv[10]))
        cons.setConstants(int(argv[11]), float(argv[12]), str(argv[13]), e, str(argv[14]), str(argv[15])) 
        sampleSize = e.getNrSamples()
        gassist = GAssist(e, argv[4], argv[5], int(argv[6]), int(argv[7]), graphPerformance)
        
        #Figure out the iteration stops for evaluation, and the max iterations.
        iterList = argv[9].split('.')
        for i in range(len(iterList)):
            iterList[i] = int(iterList[i])
        lastIter = iterList[len(iterList)-1]
        
        #Set some GAssist parameters.
        if argv[9] == 'Default':
            gassist.setTrackingIterations(sampleSize)
        else:
            gassist.setTrackingIterations(int(argv[8]))   
        gassist.setNumberOfTrials(lastIter, iterList)   
        gassist.setInitialization(str(argv[10])) #NEW - for intelligent initialization
        
        #Run the GAssist Algorithm 
        gassist.runGAssist()        
        
    else:
        print "There is no environment handling code for the specified environment"
else:
    print helpstr
    sys.exit()