"""
Name: GAssist_Sampling.py
Author: Ryan Urbanowicz - written at Dartmouth College, Hanover, NH, USA 
Note: code primarily based on Java GAssist authored by Xavier Llor <xllora@illigal.ge.uiuc.edu>
Contact: ryan.j.urbanowicz@darmouth.edu
Created: 2/22/2010
Updated: 10/15/2013
Description: Handles sampling without replacement.
---------------------------------------------------------------------------------------------------------------------------------------------------------
GAssist Version 1.0: A Python implementation of the GAssist learning classifier system algorithm implemented for SNP genetic data mining and classification tasks.
Copyright (C) 2013 Ryan Urbanowicz 
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABLILITY 
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, 
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
---------------------------------------------------------------------------------------------------------------------------------------------------------
"""

#!/usr/bin/env python

#Import modules
from random import *

class Sampling:
    def __init__(self, maxSize): 
        """  Initialize sampling objects """
        self.maxSize = maxSize
        self.sample = []
        self.num = 0
        self.initSampling()
        
        
    def initSampling(self):
        """ Initialize the sampling indexing list and maxSize parameter. """
        for i in range(0,self.maxSize):
            self.sample.append(i)
            self.num = self.maxSize
            
            
    def numSamplesLeft(self):
        """ Returns the number of samples left to draw from. """
        return self.num
    
    
    def getSample(self):
        """ Returns the next samples list position, and then removes that position from future consideration. """
        pos = randint(0,self.num-1)
        value = self.sample[pos]
        self.sample[pos] = self.sample[self.num-1]
        self.num -= 1
        if self.num == 0:
            self.initSampling()
        return value