"""
Name: GAssist_Test.py
Author: Ryan Urbanowicz - written at Dartmouth College, Hanover, NH, USA 
Note: code primarily based on Java GAssist authored by Xavier Llor <xllora@illigal.ge.uiuc.edu>
Contact: ryan.j.urbanowicz@darmouth.edu
Created: 2/22/2010
Updated: 10/15/2013
Description: A testing substitute for GAssist_Main.py to run GAssist from Eclipse(with Pydev).
---------------------------------------------------------------------------------------------------------------------------------------------------------
GAssist Version 1.0: A Python implementation of the GAssist learning classifier system algorithm implemented for SNP genetic data mining and classification tasks.
Copyright (C) 2013 Ryan Urbanowicz 
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABLILITY 
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, 
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
---------------------------------------------------------------------------------------------------------------------------------------------------------
"""

#!/usr/bin/env python

#Import modules
from GAssist import *
from GAssist_Environment import *
from GAssist_Constants import * 

def main():
    """ Runs independently from command line to test the GAssist algorithm. """
    graphPerformance = False # Built in graphing ability, currently not functional, but mechanism is in place.
    trainData = "2_1000_0_1600_0_0_CV_0_Train.txt"
    testData = "2_1000_0_1600_0_0_CV_0_Test.txt"
    outProg = "GH_GAssist_ProgressTrack"
    outPop = "GH_GAssist_PopulationOut"
    bitLength = 1    # This implementation is not yet set up to handle other rule representations, or bit encoding lengths.
    CVpartitions = 10
    trackCycles = 1
    
    # Run Parameters - User specified.
    iterInput = '20.50.100' 
    pop = 100
    wild = 0.5
    defaultClass = "0"  #auto, 0, disabled   
    init = "cw"  #'none', 'smart', 'cw'
    MDL = 1
    windows = 2
    
    #Figure out the iteration stops for evaluation, and the max iterations.
    iterList = iterInput.split('.')
    for i in range(len(iterList)):
        iterList[i] = int(iterList[i])
    lastIter = iterList[len(iterList)-1]  

    #Sets up up algorithm to be run.
    e = GAssist_Environment(trainData,testData,bitLength, init) 
    cons.setConstants(pop, wild, defaultClass, e, MDL, windows) 
    sampleSize = e.getNrSamples()
    gassist = GAssist(e, outProg, outPop, bitLength, CVpartitions, graphPerformance) 
    
    #Set some GAssist parameters.
    if trackCycles == 'Default':
        gassist.setTrackingIterations(sampleSize)
    else:
        gassist.setTrackingIterations(trackCycles) 
    gassist.setNumberOfTrials(lastIter, iterList)  
    gassist.setInitialization(init)
    #Run the GAssist Algorithm 
    gassist.runGAssist()
     
     
if __name__ == '__main__':
    main()